/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: BlueBillLookup.java,v 3e7ea5dbefcb 2010/05/17 15:10:54 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.mobile.android;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import org.openide.util.Lookup;
import org.openide.util.lookup.Lookups;
import it.tidalwave.bluebill.observation.ObservationManager;
import it.tidalwave.mobile.FileSystem;
import it.tidalwave.mobile.LazyLookup;
import it.tidalwave.mobile.LocationFinder;
import it.tidalwave.mobile.PreferencesAdapter;
import it.tidalwave.bluebill.mobile.observation.ObservationClipboard;
import it.tidalwave.bluebill.mobile.observation.BlueBillObservationManager;
import it.tidalwave.bluebill.mobile.observation.DefaultObservationPreferences;
import it.tidalwave.bluebill.mobile.observation.ObservationPreferences;
import it.tidalwave.bluebill.mobile.location.LocationPreferences;
import it.tidalwave.bluebill.mobile.taxonomy.TaxonomyPreferences;
import it.tidalwave.bluebill.mobile.network.NetworkingPreferences;
import it.tidalwave.bluebill.mobile.network.DefaultNetworkingPreferences;
import it.tidalwave.bluebill.mobile.share.SharingPreferences;
import it.tidalwave.bluebill.mobile.share.DefaultSharingPreferences;
import android.content.Context;
import android.location.LocationManager;
import android.preference.PreferenceManager;
import it.tidalwave.bluebill.mobile.DefaultGeneralPreferences;
import it.tidalwave.bluebill.mobile.GeneralPreferences;
import it.tidalwave.mobile.android.AndroidFileSystem;
import it.tidalwave.mobile.android.AndroidLocationFinder;
import it.tidalwave.mobile.android.AndroidPreferencesAdapter;
import it.tidalwave.bluebill.mobile.android.location.AndroidLocationPreferences;
import it.tidalwave.bluebill.mobile.android.taxonomy.AndroidTaxonomyPreferences;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: $
 *
 **********************************************************************************************************************/
public class BlueBillLookup extends Lookup
  {
    private final LazyLookup lookup = new LazyLookup();

    @CheckForNull
    private Lookup extraLookup = Lookup.EMPTY;

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public BlueBillLookup()
      {
        lookup.register(FileSystem.class,             AndroidFileSystem.class);
        lookup.register(GeneralPreferences.class,     DefaultGeneralPreferences.class);
        lookup.register(LocationFinder.class,         AndroidLocationFinder.class);
        lookup.register(LocationPreferences.class,    AndroidLocationPreferences.class);
        lookup.register(PreferencesAdapter.class,     AndroidPreferencesAdapter.class);
        lookup.register(TaxonomyPreferences.class,    AndroidTaxonomyPreferences.class);
        lookup.register(ObservationClipboard.class,   ObservationClipboard.class);
        lookup.register(ObservationManager.class,     BlueBillObservationManager.class);
        lookup.register(ObservationPreferences.class, DefaultObservationPreferences.class);
        lookup.register(SharingPreferences.class,     DefaultSharingPreferences.class);
        lookup.register(NetworkingPreferences.class,  DefaultNetworkingPreferences.class);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @CheckForNull
    public <T> T lookup (final @Nonnull Class<T> clazz)
      {
        final T r = extraLookup.lookup(clazz);
        return (r != null) ? r : lookup.lookup(clazz);
      }
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override @Nonnull
    public <T> Result<T> lookup (final @Nonnull Template<T> template)
      {
        final Result<T> r = extraLookup.lookup(template);
        return (r != null) ? r : lookup.lookup(template);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void setContext (final @Nonnull Context context)
      {
        extraLookup = Lookups.fixed(context,
                                    context.getAssets(),
                                    PreferenceManager.getDefaultSharedPreferences(context),
                                    (LocationManager)context.getSystemService(Context.LOCATION_SERVICE));
      }
  }
