/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: ObservationsActivity.java,v 3e7ea5dbefcb 2010/05/17 15:10:54 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.mobile.android.observation;

import javax.annotation.Nonnull;
import android.app.Dialog;
import it.tidalwave.util.logging.Logger;
import it.tidalwave.mobile.CommonUITasks;
import it.tidalwave.mobile.android.AndroidCommonUITasks;
import it.tidalwave.bluebill.mobile.observation.ObservationsUI;
import android.app.ExpandableListActivity;
import android.content.Intent;
import android.os.Bundle;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.ExpandableListView;
import android.widget.ExpandableListView.ExpandableListContextMenuInfo;
import android.widget.ImageButton;
import android.widget.TextView;
import it.tidalwave.bluebill.mobile.android.preferences.BlueBillPreferenceActivity;
import it.tidalwave.bluebill.mobile.android.R;
import it.tidalwave.bluebill.mobile.android.about.AboutActivity;
import it.tidalwave.mobile.android.ControlFlow;

/***********************************************************************************************************************
 *
 * The main activity for observations. It displays the current observation database and add / edit / etc operations can
 * be performed from here.
 *
 * @author  Fabrizio Giudici
 * @version $Id: $
 *
 **********************************************************************************************************************/
public class ObservationsActivity extends ExpandableListActivity implements ObservationsUI
  {
    private static final String CLASS = ObservationsActivity.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);

    private final AndroidObservationsUIController controller;

    private final AndroidCommonUITasks androidUI = new AndroidCommonUITasks(this);

    private TextView tvFooter;

    private View viEmpty;

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public ObservationsActivity()
      {
        controller = new AndroidObservationsUIController(this);
      }

    /*******************************************************************************************************************
     *
     * For tests.
     *
     ******************************************************************************************************************/
    protected ObservationsActivity (final @Nonnull AndroidObservationsUIController controller)
      {
        this.controller = controller;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public CommonUITasks getCommonUITasks()
      {
        return androidUI;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void setFooterText (final @Nonnull String string)
      {
        viEmpty.setVisibility(controller.getListAdapter().getGroupCount() == 0 ? View.VISIBLE : View.INVISIBLE);
        tvFooter.setText(string);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    public void onCreate (final @Nonnull Bundle savedInstanceState)
      {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.observations_activity);
        
        setListAdapter(controller.getListAdapter());
        registerForContextMenu(getExpandableListView());
        viEmpty = findViewById(R.id.empty);
        tvFooter = (TextView)findViewById(R.id.tvFooter);

        ((ImageButton)findViewById(R.id.btAdd)).setOnClickListener(new OnClickListener()
          {
            public void onClick (final @Nonnull View view)
              {
                controller.startNewObservationSequence();
              }
          });
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    protected void onResume()
      {
        controller.fireRefresh(); // e.g. we changed rendering options - FIXME: is this really needed?
        super.onResume();
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    public boolean onCreateOptionsMenu (final @Nonnull Menu menu)
      {
        getMenuInflater().inflate(R.menu.observations_options_menu, menu);
        return true;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    public void onCreateContextMenu (final @Nonnull ContextMenu menu,
                                     final @Nonnull View view,
                                     final @Nonnull ContextMenuInfo menuInfo)
      {
        final ExpandableListContextMenuInfo info = (ExpandableListContextMenuInfo)menuInfo;
        final int type = ExpandableListView.getPackedPositionType(info.packedPosition);
        int menuId = 0;

        switch (type)
          {
            case ExpandableListView.PACKED_POSITION_TYPE_GROUP:
                menuId = R.menu.observations_context_menu;
                break;

            case ExpandableListView.PACKED_POSITION_TYPE_CHILD:
                menuId = R.menu.observation_items_context_menu;
                break;
                
            default:
                return;
          }
        
        getMenuInflater().inflate(menuId, menu);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    public boolean onContextItemSelected (final @Nonnull MenuItem item)
      {
        return controller.onContextItemSelected(item);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override @Nonnull
    protected Dialog onCreateDialog (final int id)
      {
        return controller.onCreateDialog(id);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    protected void onPrepareDialog (final int id, final @Nonnull Dialog dialog)
      {
        controller.onPrepareDialog(id, dialog);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    protected void onActivityResult (final int requestCode,
                                     final int resultCode,
                                     final @Nonnull Intent data)
      {
        logger.info("onActivityResult(%d, %d, %s)", requestCode, resultCode, data);

        if ((requestCode == ControlFlow.findRequestCode(AddObservationControlFlow.class)) && (resultCode == RESULT_OK))
          {
            // TODO: would be nice to have it called by the ControlFlow.
            controller.commitNewObservation();
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    public boolean onOptionsItemSelected (final @Nonnull MenuItem item)
      {
        switch (item.getItemId())
          {
            case R.id.clearObservations:
                controller.deleteAllObservations();
                return true;

            case R.id.share:
                controller.shareObservations();
                return true;

            case R.id.browse:
                controller.browseToFactSheet();
                return true;

            case R.id.preferences:
                startActivity(new Intent(getBaseContext(), BlueBillPreferenceActivity.class));
                return true;

            case R.id.about:
                startActivity(new Intent(getBaseContext(), AboutActivity.class));
                return true;

//            case R.id.addTestData:
//                TestDataCreator.createTestData();
//                return true;

            default:
                return super.onOptionsItemSelected(item);
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void highlightLatestObservation()
      {
        try
          {
            final ExpandableListView expandableListView = getExpandableListView();
            final int groupCount = expandableListView.getCount();

            for (int i = 0; i < groupCount - 1; i++)
              {
                expandableListView.collapseGroup(i);
              }

            expandableListView.expandGroup(groupCount - 1);
          }
        catch (Exception e) // FIXME: sometime fails for IndexOutOfBoundsException in ObservationListAdapter
          {
            logger.throwing("highlightLatestObservation()", CLASS, e);
          }
      }
  }
