/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: AndroidTaxonomyPreferences.java,v 5477d0b4bb09 2010/05/22 13:18:20 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.mobile.android.taxonomy;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import org.openide.util.Lookup;
import org.openide.util.lookup.ServiceProvider;
import it.tidalwave.util.logging.Logger;
import it.tidalwave.bluebill.taxonomy.Taxon;
import it.tidalwave.bluebill.taxonomy.Taxonomy;
import it.tidalwave.bluebill.mobile.taxonomy.TaxonomyPreferences;
import it.tidalwave.bluebill.mobile.taxonomy.TaxonomyPreferencesSupport;
import android.text.Html;
import android.content.Context;
import android.content.SharedPreferences;
import android.content.SharedPreferences.OnSharedPreferenceChangeListener;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.content.res.AssetManager;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: $
 *
 **********************************************************************************************************************/
@ServiceProvider(service=TaxonomyPreferences.class)
public class AndroidTaxonomyPreferences extends TaxonomyPreferencesSupport
  {
    private static final String CLASS = AndroidTaxonomyPreferences.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);

    @Nonnull
    private final AssetManager assets = Lookup.getDefault().lookup(AssetManager.class);

    @Nonnull
    private final SharedPreferences preferences = Lookup.getDefault().lookup(SharedPreferences.class);

    @Nonnull
    private final Context context = Lookup.getDefault().lookup(Context.class);

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    private final OnSharedPreferenceChangeListener listener = new OnSharedPreferenceChangeListener()
      {
        public void onSharedPreferenceChanged (final @Nonnull SharedPreferences sharedPreferences,
                                               final @Nonnull String string)
          {
            reset();
          }
      };

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public AndroidTaxonomyPreferences()
      {
        preferences.registerOnSharedPreferenceChangeListener(listener);
        reset();
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override @Nonnull
    protected Taxonomy loadTaxonomy()
      throws Exception
      {
        final String taxonomyName = "EBNItalia2003";
        return loadTaxonomy(assets.open(taxonomyName + ".json"), "UTF-8");
//        Taxonomy taxonomy = null;
        // TODO: clean up old cached file
//        final File cachedTaxonomyFile = context.getFileStreamPath(taxonomyName + "-" + getVersion() + "a.ser");
//
//        if (cachedTaxonomyFile.exists())
//          {
//            taxonomy = loadCachedTaxonomy(cachedTaxonomyFile);
//          }
//
//        if (taxonomy == null)
//          {
//            taxonomy = loadTaxonomy(assets.open(taxonomyName + ".json"), "UTF-8");
//            writeCachedTaxonomy(cachedTaxonomyFile, taxonomy);
//          }
//
//        return taxonomy;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    private String getVersion()
      {
        try
          {
            final PackageManager manager = context.getPackageManager();
            final PackageInfo info = manager.getPackageInfo(context.getPackageName(), 0);
            return "" + info.versionCode;

          }
        catch (Exception e)
          {
            logger.warning("Couldn't find package information in PackageManager", e);
            return "0";
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @CheckForNull
    private static Taxonomy loadCachedTaxonomy (final @Nonnull File file)
      {
        try
          {
            logger.info("loadCachedTaxonomy(%s)", file);
            final long time = System.currentTimeMillis();
            final ObjectInputStream ois = new ObjectInputStream(new FileInputStream(file));
            final Taxonomy taxonomy = (Taxonomy) ois.readObject();
            ois.close();
            logger.info(">>>> taxonomy read from cache in %s msecs", System.currentTimeMillis() - time);
            return taxonomy;
          }
        catch (Throwable e) // See BBMA-44
          {
            logger.warning("Couldn't read the taxonomy from %s: %s", file, e);
          }

        return null;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    private static void writeCachedTaxonomy (final @Nonnull File file, final @Nonnull Taxonomy taxonomy)
      {
        try
          {
            logger.info("writeCachedTaxonomy(%s, %s)", file, taxonomy.getDisplayName());
            final long time = System.currentTimeMillis();
            final ObjectOutputStream oos = new ObjectOutputStream(new FileOutputStream(file));
            oos.writeObject(taxonomy);
            oos.close();
            logger.info(">>>> taxonomy written to cache in %s msecs", System.currentTimeMillis() - time);
          }
        catch (Throwable e) // See BBMA-44
          {
            logger.warning("Couldn't write the taxonomy to %s: %s", file, e);
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override @Nonnull
    public CharSequence formatAsHtml (final @Nonnull Taxon taxon, @Nonnull String extra)
      {
        return Html.fromHtml(super.formatAsHtml(taxon, extra).toString());
      }
  }
