/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: AndroidPickTaxonController.java,v 3e7ea5dbefcb 2010/05/17 15:10:54 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.mobile.android.taxonomy.browser;

import javax.annotation.Nonnegative;
import javax.annotation.Nonnull;
import java.util.Collections;
import java.util.List;
import org.openide.util.Lookup;
import it.tidalwave.util.logging.Logger;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.bluebill.taxonomy.Taxon;
import it.tidalwave.bluebill.taxonomy.Taxonomy.Type;
import it.tidalwave.mobile.android.ControlFlow;
import it.tidalwave.bluebill.mobile.observation.ObservationClipboard;
import it.tidalwave.bluebill.mobile.taxonomy.TaxonomyPreferences;
import it.tidalwave.bluebill.mobile.android.taxonomy.TaxonomyAdapter;
import static it.tidalwave.bluebill.mobile.android.taxonomy.TaxonIntentHelper.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: $
 *
 **********************************************************************************************************************/
public class AndroidPickTaxonController
  {
    private static final String CLASS = AndroidPickTaxonController.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);

    private final Lookup lookup = Lookup.getDefault();

    private final TaxonomyPreferences preferences = lookup.lookup(TaxonomyPreferences.class);

    private final ObservationClipboard observationClipboard = lookup.lookup(ObservationClipboard.class);

    private TaxonomyAdapter taxonHistoryAdapter;

    private final PickTaxonActivity activity;

    /*******************************************************************************************************************
     *
     *
     *
     ******************************************************************************************************************/
    public AndroidPickTaxonController (final @Nonnull PickTaxonActivity activity)
      {
        this.activity = activity;
      }

    /*******************************************************************************************************************
     *
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public TaxonomyAdapter getTaxonHistoryAdapter()
      {
        return taxonHistoryAdapter;
      }

    /*******************************************************************************************************************
     *
     *
     *
     ******************************************************************************************************************/
    public void pickHistoricTaxon (final @Nonnegative int position)
      {
        try
          {
            final Taxon taxon = taxonHistoryAdapter.getItem(position);
            ControlFlow.from(activity).toNextStep(intentFor(taxon));
          }
        catch (NotFoundException e)
          {
            throw new RuntimeException(e);
          }
      }

    /*******************************************************************************************************************
     *
     *
     *
     ******************************************************************************************************************/
    public void startBrowsingByType (final @Nonnull Type type)
      {
        ControlFlow.from(activity).toNextStep(intentFor(type), ControlFlow.USE_INTENT_FILTER);
      }

    /*******************************************************************************************************************
     *
     *
     *
     ******************************************************************************************************************/
    public void bindTaxons()
      {
        final List<Taxon> historicTaxons = observationClipboard.getHistoricTaxons();
        final long time = System.currentTimeMillis();
        Collections.sort(historicTaxons, preferences.getTaxonComparator());
        logger.info("taxons sorted in %d msec", System.currentTimeMillis() - time);
        taxonHistoryAdapter = new TaxonomyAdapter(activity, preferences, historicTaxons);
        activity.updateList();
      }

    /*******************************************************************************************************************
     *
     *
     *
     ******************************************************************************************************************/
    public void clearTaxonHistory()
      {
        observationClipboard.clearTaxonHistory();
        bindTaxons();
      }
  }
